@@ -46,6 +46,8 @@
 #include <qv4runtime_p.h>
 #include <qv4variantobject_p.h>
 #include "qv4string_p.h"
+#include "qv4jscall_p.h"
+#include <qv4symbol_p.h>
 
 #include <qstack.h>
 #include <qstringlist.h>
@@ -123,12 +125,13 @@ enum {
 bool JsonParser::eatSpace()
 {
     while (json < end) {
-        if (*json > Space)
+        const char16_t ch = json->unicode();
+        if (ch > Space)
             break;
-        if (*json != Space &&
-            *json != Tab &&
-            *json != LineFeed &&
-            *json != Return)
+        if (ch != Space &&
+            ch != Tab &&
+            ch != LineFeed &&
+            ch != Return)
             break;
         ++json;
     }
@@ -138,7 +141,7 @@ bool JsonParser::eatSpace()
 QChar JsonParser::nextToken()
 {
     if (!eatSpace())
-        return 0;
+        return u'\0';
     QChar token = *json++;
     switch (token.unicode()) {
     case BeginArray:
@@ -151,7 +154,7 @@ QChar JsonParser::nextToken()
     case Quote:
         break;
     default:
-        token = 0;
+        token = u'\0';
         break;
     }
     return token;
@@ -214,21 +217,21 @@ ReturnedValue JsonParser::parseObject()
     ScopedObject o(scope, engine->newObject());
 
     QChar token = nextToken();
-    while (token == Quote) {
+    while (token.unicode() == Quote) {
         if (!parseMember(o))
             return Encode::undefined();
         token = nextToken();
-        if (token != ValueSeparator)
+        if (token.unicode() != ValueSeparator)
             break;
         token = nextToken();
-        if (token == EndObject) {
+        if (token.unicode() == EndObject) {
             lastError = QJsonParseError::MissingObject;
             return Encode::undefined();
         }
     }
 
     DEBUG << "end token=" << token;
-    if (token != EndObject) {
+    if (token.unicode() != EndObject) {
         lastError = QJsonParseError::UnterminatedObject;
         return Encode::undefined();
     }
@@ -251,7 +254,7 @@ bool JsonParser::parseMember(Object *o)
     if (!parseString(&key))
         return false;
     QChar token = nextToken();
-    if (token != NameSeparator) {
+    if (token.unicode() != NameSeparator) {
         lastError = QJsonParseError::MissingNameSeparator;
         return false;
     }
@@ -259,11 +262,12 @@ bool JsonParser::parseMember(Object *o)
     if (!parseValue(val))
         return false;
 
-    ScopedString s(scope, engine->newIdentifier(key));
-    uint idx = s->asArrayIndex();
-    if (idx < UINT_MAX) {
-        o->putIndexed(idx, val);
+    ScopedString s(scope, engine->newString(key));
+    PropertyKey skey = s->toPropertyKey();
+    if (skey.isArrayIndex()) {
+        o->put(skey.asArrayIndex(), val);
     } else {
+        // avoid trouble with properties named __proto__
         o->insertMember(s, val);
     }
 
@@ -289,7 +293,7 @@ ReturnedValue JsonParser::parseArray()
         lastError = QJsonParseError::UnterminatedArray;
         return Encode::undefined();
     }
-    if (*json == EndArray) {
+    if (json->unicode() == EndArray) {
         nextToken();
     } else {
         uint index = 0;
@@ -299,9 +303,9 @@ ReturnedValue JsonParser::parseArray()
                 return Encode::undefined();
             array->arraySet(index, val);
             QChar token = nextToken();
-            if (token == EndArray)
+            if (token.unicode() == EndArray)
                 break;
-            else if (token != ValueSeparator) {
+            else if (token.unicode() != ValueSeparator) {
                 if (!eatSpace())
                     lastError = QJsonParseError::UnterminatedArray;
                 else
@@ -329,46 +333,46 @@ bool JsonParser::parseValue(Value *val)
     BEGIN << "parse Value" << *json;
 
     switch ((json++)->unicode()) {
-    case 'n':
+    case u'n':
         if (end - json < 3) {
             lastError = QJsonParseError::IllegalValue;
             return false;
         }
-        if (*json++ == 'u' &&
-            *json++ == 'l' &&
-            *json++ == 'l') {
-            *val = Primitive::nullValue();
+        if (*json++ == u'u' &&
+            *json++ == u'l' &&
+            *json++ == u'l') {
+            *val = Value::nullValue();
             DEBUG << "value: null";
             END;
             return true;
         }
         lastError = QJsonParseError::IllegalValue;
         return false;
-    case 't':
+    case u't':
         if (end - json < 3) {
             lastError = QJsonParseError::IllegalValue;
             return false;
         }
-        if (*json++ == 'r' &&
-            *json++ == 'u' &&
-            *json++ == 'e') {
-            *val = Primitive::fromBoolean(true);
+        if (*json++ == u'r' &&
+            *json++ == u'u' &&
+            *json++ == u'e') {
+            *val = Value::fromBoolean(true);
             DEBUG << "value: true";
             END;
             return true;
         }
         lastError = QJsonParseError::IllegalValue;
         return false;
-    case 'f':
+    case u'f':
         if (end - json < 4) {
             lastError = QJsonParseError::IllegalValue;
             return false;
         }
-        if (*json++ == 'a' &&
-            *json++ == 'l' &&
-            *json++ == 's' &&
-            *json++ == 'e') {
-            *val = Primitive::fromBoolean(false);
+        if (*json++ == u'a' &&
+            *json++ == u'l' &&
+            *json++ == u's' &&
+            *json++ == u'e') {
+            *val = Value::fromBoolean(false);
             DEBUG << "value: false";
             END;
             return true;
@@ -440,32 +444,32 @@ bool JsonParser::parseNumber(Value *val)
     bool isInt = true;
 
     // minus
-    if (json < end && *json == '-')
+    if (json < end && *json == u'-')
         ++json;
 
     // int = zero / ( digit1-9 *DIGIT )
-    if (json < end && *json == '0') {
+    if (json < end && *json == u'0') {
         ++json;
     } else {
-        while (json < end && *json >= '0' && *json <= '9')
+        while (json < end && *json >= u'0' && *json <= u'9')
             ++json;
     }
 
     // frac = decimal-point 1*DIGIT
-    if (json < end && *json == '.') {
+    if (json < end && *json == u'.') {
         isInt = false;
         ++json;
-        while (json < end && *json >= '0' && *json <= '9')
+        while (json < end && *json >= u'0' && *json <= u'9')
             ++json;
     }
 
     // exp = e [ minus / plus ] 1*DIGIT
-    if (json < end && (*json == 'e' || *json == 'E')) {
+    if (json < end && (*json == u'e' || *json == u'E')) {
         isInt = false;
         ++json;
-        if (json < end && (*json == '-' || *json == '+'))
+        if (json < end && (*json == u'-' || *json == u'+'))
             ++json;
-        while (json < end && *json >= '0' && *json <= '9')
+        while (json < end && *json >= u'0' && *json <= u'9')
             ++json;
     }
 
@@ -476,7 +480,7 @@ bool JsonParser::parseNumber(Value *val)
         bool ok;
         int n = number.toInt(&ok);
         if (ok && n < (1<<25) && n > -(1<<25)) {
-            *val = Primitive::fromInt32(n);
+            *val = Value::fromInt32(n);
             END;
             return true;
         }
@@ -491,7 +495,7 @@ bool JsonParser::parseNumber(Value *val)
         return false;
     }
 
-    * val = Primitive::fromDouble(d);
+    * val = Value::fromDouble(d);
 
     END;
     return true;
@@ -523,12 +527,12 @@ static inline bool addHexDigit(QChar digit, uint *result)
 {
     ushort d = digit.unicode();
     *result <<= 4;
-    if (d >= '0' && d <= '9')
-        *result |= (d - '0');
-    else if (d >= 'a' && d <= 'f')
-        *result |= (d - 'a') + 10;
-    else if (d >= 'A' && d <= 'F')
-        *result |= (d - 'A') + 10;
+    if (d >= u'0' && d <= u'9')
+        *result |= (d - u'0');
+    else if (d >= u'a' && d <= u'f')
+        *result |= (d - u'a') + 10;
+    else if (d >= u'A' && d <= u'F')
+        *result |= (d - u'A') + 10;
     else
         return false;
     return true;
@@ -543,23 +547,23 @@ static inline bool scanEscapeSequence(const QChar *&json, const QChar *end, uint
     DEBUG << "scan escape";
     uint escaped = (json++)->unicode();
     switch (escaped) {
-    case '"':
+    case u'"':
         *ch = '"'; break;
-    case '\\':
+    case u'\\':
         *ch = '\\'; break;
-    case '/':
+    case u'/':
         *ch = '/'; break;
-    case 'b':
+    case u'b':
         *ch = 0x8; break;
-    case 'f':
+    case u'f':
         *ch = 0xc; break;
-    case 'n':
+    case u'n':
         *ch = 0xa; break;
-    case 'r':
+    case u'r':
         *ch = 0xd; break;
-    case 't':
+    case u't':
         *ch = 0x9; break;
-    case 'u': {
+    case u'u': {
         *ch = 0;
         if (json > end - 4)
             return false;
@@ -582,9 +586,9 @@ bool JsonParser::parseString(QString *string)
     BEGIN << "parse string stringPos=" << json;
 
     while (json < end) {
-        if (*json == '"')
+        if (*json == u'"')
             break;
-        else if (*json == '\\') {
+        else if (*json == u'\\') {
             uint ch = 0;
             if (!scanEscapeSequence(json, end, &ch)) {
                 lastError = QJsonParseError::IllegalEscapeSequence;
@@ -633,10 +637,10 @@ struct Stringify
         return false;
     }
 
-    Stringify(ExecutionEngine *e) : v4(e), replacerFunction(0), propertyList(0), propertyListSize(0) {}
+    Stringify(ExecutionEngine *e) : v4(e), replacerFunction(nullptr), propertyList(nullptr), propertyListSize(0) {}
 
     QString Str(const QString &key, const Value &v);
-    QString JA(ArrayObject *a);
+    QString JA(Object *a);
     QString JO(Object *o);
 
     QString makeMember(const QString &key, const Value &v);
@@ -647,103 +651,107 @@ static QString quote(const QString &str)
     QString product;
     const int length = str.length();
     product.reserve(length + 2);
-    product += QLatin1Char('"');
+    product += u'"';
     for (int i = 0; i < length; ++i) {
         QChar c = str.at(i);
         switch (c.unicode()) {
-        case '"':
+        case u'"':
             product += QLatin1String("\\\"");
             break;
-        case '\\':
+        case u'\\':
             product += QLatin1String("\\\\");
             break;
-        case '\b':
+        case u'\b':
             product += QLatin1String("\\b");
             break;
-        case '\f':
+        case u'\f':
             product += QLatin1String("\\f");
             break;
-        case '\n':
+        case u'\n':
             product += QLatin1String("\\n");
             break;
-        case '\r':
+        case u'\r':
             product += QLatin1String("\\r");
             break;
-        case '\t':
+        case u'\t':
             product += QLatin1String("\\t");
             break;
         default:
             if (c.unicode() <= 0x1f) {
                 product += QLatin1String("\\u00");
-                product += (c.unicode() > 0xf ? QLatin1Char('1') : QLatin1Char('0')) +
+                product += (c.unicode() > 0xf ? u'1' : u'0') +
                         QLatin1Char("0123456789abcdef"[c.unicode() & 0xf]);
             } else {
                 product += c;
             }
         }
     }
-    product += QLatin1Char('"');
+    product += u'"';
     return product;
 }
 
 QString Stringify::Str(const QString &key, const Value &v)
 {
     Scope scope(v4);
-    scope.result = v;
 
-    ScopedObject o(scope, scope.result);
+    ScopedValue value(scope, v);
+    ScopedObject o(scope, value);
     if (o) {
         ScopedString s(scope, v4->newString(QStringLiteral("toJSON")));
         ScopedFunctionObject toJSON(scope, o->get(s));
         if (!!toJSON) {
-            ScopedCallData callData(scope, 1);
-            callData->thisObject = scope.result;
-            callData->args[0] = v4->newString(key);
-            toJSON->call(scope, callData);
+            JSCallData jsCallData(scope, 1);
+            *jsCallData->thisObject = value;
+            jsCallData->args[0] = v4->newString(key);
+            value = toJSON->call(jsCallData);
+            if (v4->hasException)
+                return QString();
         }
     }
 
     if (replacerFunction) {
         ScopedObject holder(scope, v4->newObject());
-        holder->put(scope.engine->id_empty(), scope.result);
-        ScopedCallData callData(scope, 2);
-        callData->args[0] = v4->newString(key);
-        callData->args[1] = scope.result;
-        callData->thisObject = holder;
-        replacerFunction->call(scope, callData);
+        holder->put(scope.engine->id_empty(), value);
+        JSCallData jsCallData(scope, 2);
+        jsCallData->args[0] = v4->newString(key);
+        jsCallData->args[1] = value;
+        *jsCallData->thisObject = holder;
+        value = replacerFunction->call(jsCallData);
+        if (v4->hasException)
+            return QString();
     }
 
-    o = scope.result.asReturnedValue();
+    o = value->asReturnedValue();
     if (o) {
         if (NumberObject *n = o->as<NumberObject>())
-            scope.result = Encode(n->value());
+            value = Encode(n->value());
         else if (StringObject *so = o->as<StringObject>())
-            scope.result = so->d()->string;
+            value = so->d()->string;
         else if (BooleanObject *b = o->as<BooleanObject>())
-            scope.result = Encode(b->value());
+            value = Encode(b->value());
     }
 
-    if (scope.result.isNull())
+    if (value->isNull())
         return QStringLiteral("null");
-    if (scope.result.isBoolean())
-        return scope.result.booleanValue() ? QStringLiteral("true") : QStringLiteral("false");
-    if (String *s = scope.result.stringValue())
-        return quote(s->toQString());
+    if (value->isBoolean())
+        return value->booleanValue() ? QStringLiteral("true") : QStringLiteral("false");
+    if (value->isString())
+        return quote(value->stringValue()->toQString());
 
-    if (scope.result.isNumber()) {
-        double d = scope.result.toNumber();
-        return std::isfinite(d) ? scope.result.toQString() : QStringLiteral("null");
+    if (value->isNumber()) {
+        double d = value->toNumber();
+        return std::isfinite(d) ? value->toQString() : QStringLiteral("null");
     }
 
-    if (const QV4::VariantObject *v = scope.result.as<QV4::VariantObject>()) {
-        return v->d()->data().toString();
+    if (const QV4::VariantObject *v = value->as<QV4::VariantObject>()) {
+        return quote(v->d()->data().toString());
     }
 
-    o = scope.result.asReturnedValue();
+    o = value->asReturnedValue();
     if (o) {
         if (!o->as<FunctionObject>()) {
-            if (o->as<ArrayObject>()) {
-                return JA(static_cast<ArrayObject *>(o.getPointer()));
+            if (o->isArrayLike()) {
+                return JA(o.getPointer());
             } else {
                 return JO(o);
             }
@@ -757,9 +765,9 @@ QString Stringify::makeMember(const QString &key, const Value &v)
 {
     QString strP = Str(key, v);
     if (!strP.isEmpty()) {
-        QString member = quote(key) + QLatin1Char(':');
+        QString member = quote(key) + u':';
         if (!gap.isEmpty())
-            member += QLatin1Char(' ');
+            member += u' ';
         member += strP;
         return member;
     }
@@ -814,11 +822,11 @@ QString Stringify::JO(Object *o)
     if (partial.isEmpty()) {
         result = QStringLiteral("{}");
     } else if (gap.isEmpty()) {
-        result = QLatin1Char('{') + partial.join(QLatin1Char(',')) + QLatin1Char('}');
+        result = u'{' + partial.join(u',') + u'}';
     } else {
         QString separator = QLatin1String(",\n") + indent;
-        result = QLatin1String("{\n") + indent + partial.join(separator) + QLatin1Char('\n')
-                 + stepback + QLatin1Char('}');
+        result = QLatin1String("{\n") + indent + partial.join(separator) + u'\n'
+                 + stepback + u'}';
     }
 
     indent = stepback;
@@ -826,7 +834,7 @@ QString Stringify::JO(Object *o)
     return result;
 }
 
-QString Stringify::JA(ArrayObject *a)
+QString Stringify::JA(Object *a)
 {
     if (stackContains(a)) {
         v4->throwTypeError();
@@ -845,7 +853,7 @@ QString Stringify::JA(ArrayObject *a)
     ScopedValue v(scope);
     for (uint i = 0; i < len; ++i) {
         bool exists;
-        v = a->getIndexed(i, &exists);
+        v = a->get(i, &exists);
         if (!exists) {
             partial += QStringLiteral("null");
             continue;
@@ -860,10 +868,10 @@ QString Stringify::JA(ArrayObject *a)
     if (partial.isEmpty()) {
         result = QStringLiteral("[]");
     } else if (gap.isEmpty()) {
-        result = QLatin1Char('[') + partial.join(QLatin1Char(',')) + QLatin1Char(']');
+        result = u'[' + partial.join(u',') + u']';
     } else {
         QString separator = QLatin1String(",\n") + indent;
-        result = QLatin1String("[\n") + indent + partial.join(separator) + QLatin1Char('\n') + stepback + QLatin1Char(']');
+        result = QLatin1String("[\n") + indent + partial.join(separator) + u'\n' + stepback + u']';
     }
 
     indent = stepback;
@@ -880,31 +888,36 @@ void Heap::JsonObject::init()
 
     o->defineDefaultProperty(QStringLiteral("parse"), QV4::JsonObject::method_parse, 2);
     o->defineDefaultProperty(QStringLiteral("stringify"), QV4::JsonObject::method_stringify, 3);
+    ScopedString json(scope, scope.engine->newString(QStringLiteral("JSON")));
+    o->defineReadonlyConfigurableProperty(scope.engine->symbol_toStringTag(), json);
 }
 
 
-void JsonObject::method_parse(const BuiltinFunction *, Scope &scope, CallData *callData)
+ReturnedValue JsonObject::method_parse(const FunctionObject *b, const Value *, const Value *argv, int argc)
 {
-    ScopedValue v(scope, callData->argument(0));
-    QString jtext = v->toQString();
+    ExecutionEngine *v4 = b->engine();
+    QString jtext;
+    if (argc > 0)
+        jtext = argv[0].toQString();
 
     DEBUG << "parsing source = " << jtext;
-    JsonParser parser(scope.engine, jtext.constData(), jtext.length());
+    JsonParser parser(v4, jtext.constData(), jtext.length());
     QJsonParseError error;
-    ScopedValue result(scope, parser.parse(&error));
+    ReturnedValue result = parser.parse(&error);
     if (error.error != QJsonParseError::NoError) {
         DEBUG << "parse error" << error.errorString();
-        RETURN_RESULT(scope.engine->throwSyntaxError(QStringLiteral("JSON.parse: Parse error")));
+        RETURN_RESULT(v4->throwSyntaxError(QStringLiteral("JSON.parse: Parse error")));
     }
 
-    scope.result = result;
+    return result;
 }
 
-void JsonObject::method_stringify(const BuiltinFunction *, Scope &scope, CallData *callData)
+ReturnedValue JsonObject::method_stringify(const FunctionObject *b, const Value *, const Value *argv, int argc)
 {
+    Scope scope(b);
     Stringify stringify(scope.engine);
 
-    ScopedObject o(scope, callData->argument(1));
+    ScopedObject o(scope, argc > 1 ? argv[1] : Value::undefinedValue());
     if (o) {
         stringify.replacerFunction = o->as<FunctionObject>();
         if (o->isArrayObject()) {
@@ -912,15 +925,15 @@ void JsonObject::method_stringify(const BuiltinFunction *, Scope &scope, CallDat
             stringify.propertyList = static_cast<QV4::String *>(scope.alloc(arrayLen));
             for (uint i = 0; i < arrayLen; ++i) {
                 Value *v = stringify.propertyList + i;
-                *v = o->getIndexed(i);
+                *v = o->get(i);
                 if (v->as<NumberObject>() || v->as<StringObject>() || v->isNumber())
                     *v = v->toString(scope.engine);
                 if (!v->isString()) {
-                    v->setM(0);
+                    v->setM(nullptr);
                 } else {
                     for (uint j = 0; j <i; ++j) {
                         if (stringify.propertyList[j].m() == v->m()) {
-                            v->setM(0);
+                            v->setM(nullptr);
                             break;
                         }
                     }
@@ -929,24 +942,24 @@ void JsonObject::method_stringify(const BuiltinFunction *, Scope &scope, CallDat
         }
     }
 
-    ScopedValue s(scope, callData->argument(2));
+    ScopedValue s(scope, argc > 2 ? argv[2] : Value::undefinedValue());
     if (NumberObject *n = s->as<NumberObject>())
         s = Encode(n->value());
     else if (StringObject *so = s->as<StringObject>())
         s = so->d()->string;
 
     if (s->isNumber()) {
-        stringify.gap = QString(qMin(10, (int)s->toInteger()), ' ');
+        stringify.gap = QString(qMin(10, (int)s->toInteger()), u' ');
     } else if (String *str = s->stringValue()) {
         stringify.gap = str->toQString().left(10);
     }
 
 
-    ScopedValue arg0(scope, callData->argument(0));
+    ScopedValue arg0(scope, argc ? argv[0] : Value::undefinedValue());
     QString result = stringify.Str(QString(), arg0);
     if (result.isEmpty() || scope.engine->hasException)
         RETURN_UNDEFINED();
-    scope.result = scope.engine->newString(result);
+    return Encode(scope.engine->newString(result));
 }
 
 
@@ -1074,7 +1087,7 @@ QJsonArray JsonObject::toJsonArray(const ArrayObject *a, V4ObjectSet &visitedObj
     ScopedValue v(scope);
     quint32 length = a->getLength();
     for (quint32 i = 0; i < length; ++i) {
-        v = a->getIndexed(i);
+        v = a->get(i);
         if (v->as<FunctionObject>())
             v = Encode::null();
         result.append(toJsonValue(v, visitedObjects));
