@@ -117,11 +117,13 @@ void tst_QQuickAccessible::cleanup()
 {
     const EventList list = QTestAccessibility::events();
     if (!list.isEmpty()) {
-        qWarning("%d accessibility event(s) were not handled in testfunction '%s':", list.count(),
-                 QString(QTest::currentTestFunction()).toLatin1().constData());
+        qWarning().noquote() << list.count()
+                             << "accessibility event(s) were not handled in testfunction '"
+                             << QTest::currentTestFunction() << "':";
         for (int i = 0; i < list.count(); ++i)
-            qWarning(" %d: Object: %p Event: '%s' Child: %d", i + 1, list.at(i)->object(),
-                     qAccessibleEventString(list.at(i)->type()), list.at(i)->child());
+            qWarning().noquote() << " " << (i + 1) << ": Object: " << list.at(i)->object()
+                       << "Event: '" << qAccessibleEventString(list.at(i)->type())
+                       << "' Child: " << list.at(i)->child();
     }
     QTestAccessibility::clearEvents();
 }
@@ -169,9 +171,22 @@ void tst_QQuickAccessible::quickAttachedProperties()
         delete object;
     }
 
-    // Attached property
+    // Attaching to non-item
     {
         QObject parent;
+        QTest::ignoreMessage(QtWarningMsg, "<Unknown File>: QML QtObject: Accessible must be attached to an Item");
+        QQuickAccessibleAttached *attachedObj = new QQuickAccessibleAttached(&parent);
+
+        QCOMPARE(attachedObj->ignored(), false);
+        attachedObj->setIgnored(true);
+        QCOMPARE(attachedObj->ignored(), false);
+        attachedObj->setIgnored(false);
+        QCOMPARE(attachedObj->ignored(), false);
+    }
+
+    // Attached property
+    {
+        QQuickItem parent;
         QQuickAccessibleAttached *attachedObj = new QQuickAccessibleAttached(&parent);
 
         attachedObj->name();
@@ -193,9 +208,11 @@ void tst_QQuickAccessible::quickAttachedProperties()
             QCOMPARE(p.isNull(), false);
             QCOMPARE(p.toInt(), int(QAccessible::PushButton));
             p = attachedObject->property("name");
-            QCOMPARE(p.isNull(), true);
+            QCOMPARE(p.typeId(), QMetaType::QString);
+            QVERIFY2(p.value<QString>().isEmpty(), QTest::toString(p));
             p = attachedObject->property("description");
-            QCOMPARE(p.isNull(), true);
+            QCOMPARE(p.typeId(), QMetaType::QString);
+            QVERIFY2(p.value<QString>().isEmpty(), QTest::toString(p));
             QCOMPARE(attachedObject->wasNameExplicitlySet(), false);
         }
         delete object;
@@ -368,8 +385,8 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QCOMPARE(text2->rect().y(), item->rect().y() + 40);
     QCOMPARE(text2->role(), QAccessible::StaticText);
     QCOMPARE(item->indexOfChild(text2), 1);
-    QCOMPARE(text2->state().editable, 0u);
-    QCOMPARE(text2->state().readOnly, 1);
+    QVERIFY(!text2->state().editable);
+    QVERIFY(text2->state().readOnly);
 
     QCOMPARE(iface->indexOfChild(text2), -1);
     QCOMPARE(text2->indexOfChild(item), -1);
@@ -379,10 +396,10 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QVERIFY(textInput);
     QCOMPARE(textInput->childCount(), 0);
     QCOMPARE(textInput->role(), QAccessible::EditableText);
-    QCOMPARE(textInput->state().editable, 1);
-    QCOMPARE(textInput->state().readOnly, 0);
-    QCOMPARE(textInput->state().multiLine, 0);
-    QCOMPARE(textInput->state().focusable, 1);
+    QVERIFY(textInput->state().editable);
+    QVERIFY(!textInput->state().readOnly);
+    QVERIFY(!textInput->state().multiLine);
+    QVERIFY(textInput->state().focusable);
     QCOMPARE(textInput->text(QAccessible::Value), "A text input");
     auto textInterface = textInput->textInterface();
     QVERIFY(textInterface);
@@ -398,9 +415,9 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QVERIFY(textEdit);
     QCOMPARE(textEdit->childCount(), 0);
     QCOMPARE(textEdit->role(), QAccessible::EditableText);
-    QCOMPARE(textEdit->state().editable, 1);
-    QCOMPARE(textEdit->state().readOnly, 0);
-    QCOMPARE(textEdit->state().focusable, 1);
+    QVERIFY(textEdit->state().editable);
+    QVERIFY(!textEdit->state().readOnly);
+    QVERIFY(textEdit->state().focusable);
     QCOMPARE(textEdit->text(QAccessible::Value), "A multi-line text edit\nTesting Accessibility.");
     auto textEditTextInterface = textEdit->textInterface();
     QVERIFY(textEditTextInterface);
@@ -410,7 +427,7 @@ void tst_QQuickAccessible::basicPropertiesTest()
     textEdit->setText(QAccessible::Value, newText);
     QCOMPARE(textEdit->text(QAccessible::Value), newText);
     QEXPECT_FAIL("", "multi line is not implemented", Continue);
-    QCOMPARE(textInput->state().multiLine, 1);
+    QVERIFY(textInput->state().multiLine);
 
     // Text "Hello 3"
     QAccessibleInterface *text3 = item->child(4);
@@ -419,22 +436,22 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QCOMPARE(text3->text(QAccessible::Name), QLatin1String("Hello 3"));
     QCOMPARE(text3->role(), QAccessible::StaticText);
     QCOMPARE(item->indexOfChild(text3), 4);
-    QCOMPARE(text3->state().editable, 0);
-    QCOMPARE(text3->state().readOnly, 0);
+    QVERIFY(!text3->state().editable);
+    QVERIFY(!text3->state().readOnly);
     // test implicit state values due to role change
     QQuickAccessibleAttached *attached = QQuickAccessibleAttached::attachedProperties(text3->object());
     attached->setRole(QAccessible::StaticText);
     QCOMPARE(text3->role(), QAccessible::StaticText);
-    QCOMPARE(text3->state().readOnly, 1);
+    QVERIFY(text3->state().readOnly);
 
     // see if implicit changes back
     attached->setRole(QAccessible::EditableText);
     QEXPECT_FAIL("", "EditableText does not implicitly set readOnly to false", Continue);
-    QCOMPARE(text3->state().readOnly, 0);
+    QVERIFY(!text3->state().readOnly);
     // explicitly set state
     attached->set_readOnly(false);
     attached->setRole(QAccessible::StaticText);
-    QCOMPARE(text3->state().readOnly, 0);
+    QVERIFY(!text3->state().readOnly);
 
     delete window;
     QTestAccessibility::clearEvents();
